/*
Copyright (c) 2004 EarthLink, Inc.
All rights reserved.

FPeer Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created on Mar 29, 2004
 *
 */
package net.earthlink.research.p2p;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * @author Mark Petrovic
 *
 * A NetherHost is defined as a host that I know about, but to which
 * I am not subscribed, although that host may be subscribed to me 
 * (i.e., one of my Subscribers).  
 */
public class NetherHosts extends Vector {

   private String        myIPAddress;
   private Logger        logger;

   /**
    * 
    *
    */
   public NetherHosts() {
      myIPAddress   = FPeer.fpeer.rtProps.getProperty("javax.sip.IP_ADDRESS");
      logger        = Logger.getLogger(this.getClass().getName());
      readPeerFile();
   }

   /**
    * 
    * @param addr
    */
   public void add
      (String addr) {
      if( ! (isNetherHost(addr) || FPeer.fpeer.subscriptions.isSubscription(addr) || myIPAddress.equals(addr) ) ) {
         super.add(addr);
         FPeer.fpeer.updateWindowManager();
         logger.debug("netherHost added =" + addr);

      }
   }

   /**
    * 
    * @param addr
    */
   public void remove (String addr) {
      Vector v;

      v = new Vector();
      for(Iterator i = iterator(); i.hasNext(); ) {
         String s;
         s = (String )i.next();
         if( s.equals(addr)) {
            v.add(s);
            break;
         }
      }
      
      // Avoid java.util.ConcurrentModificationException
      for(Iterator i = v.iterator(); i.hasNext(); ) {
         super.remove(i.next());
         logger.warn("netherHost removed = " + addr);
      }
      FPeer.fpeer.updateWindowManager();
   }

   /**
    * 
    * @param addr
    * @return
    */
   public boolean isNetherHost(String addr) {
      boolean b = false;

      for(Iterator i = iterator(); i.hasNext(); ) {
         String s;
         s = (String )i.next();
         b = b || s.equals(addr);
         if (b)
            break;
      }
      return b;
   }

   /**
    * 
    *
    */
   private void readPeerFile() {
      BufferedReader br;
      String         line;
      String         peerFile;

      peerFile = FPeer.fpeer.rtProps.getProperty("net.earthlink.research.p2p.peersFile", "peers");
      try {
         br = new BufferedReader( new FileReader(peerFile) );
         while( (line = br.readLine()) != null ) {
            line = line.trim();
            if( ! myIPAddress.equals(line) ) {
               super.add(line);
            }
         }
      } catch (FileNotFoundException e) {
         logger.fatal("No peers file = " + peerFile + ", exiting.");
         System.exit(0);
      } catch (IOException e) {
         logger.fatal("IO exception, exiting.");
         System.exit(0);
      }

      logger.info("readPeerFile() netherHosts = " + this);

   }


}
