/*
Copyright (c) 2004 EarthLink, Inc.
All rights reserved.

FPeer Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// ShareList.java

package net.earthlink.research.p2p;

import java.io.File;

import java.util.Iterator;
import java.util.Vector;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import org.apache.log4j.Logger;

import org.apache.regexp.RE;
import org.apache.regexp.RESyntaxException;

/**
*
* @author Mark Petrovic
*/
public class ShareList {

   private  String[] children;
   private  String   sharedDirectory;
   private  File     dir;
   private  Logger   logger;

   private  Vector   sharedFiles;

   //---------------------------------------
   public ShareList(String sharedDirectory) {
      logger      = Logger.getLogger(this.getClass().getName());

      this.sharedDirectory = sharedDirectory;

      dir         = new File(this.sharedDirectory);
      children    = dir.list();
      sharedFiles = new Vector();

      logger.warn("Building shared file cache...");
      for( int i = 0; i < children.length; ++i ) {
         logger.debug("adding to file cache:  " + children[i]);
         sharedFiles.add(new SharedFile(this.sharedDirectory, children[i]));
      }
   }

   public String toXML() {
      Document     doc;
      Element      rootElement;
      Element      element;

      rootElement = new Element("sharelist");
      doc         = new Document(rootElement);
      for( int i = 0; i < children.length; ++i ) {
         element = new Element("file");
         element.addContent(children[i]);
         rootElement.addContent(element);
      }
      return new XMLOutputter(Format.getPrettyFormat()).outputString(doc);
   }

   public boolean hasHits(String regex) {
      String  fName;
      boolean matches;
      RE      re;

      matches = false;
      try {
         re      = new RE(regex, RE.MATCH_CASEINDEPENDENT);
         for( Iterator i = sharedFiles.iterator(); i.hasNext();  ) {
            fName = ( (SharedFile )i.next() ).fileName;
            if ( re.match(fName) ) {
               matches = true;
               break;
            }
         }
      }
      catch (RESyntaxException e) {
         logger.error("RE syntax exception:  " + regex);   
      }
      return matches;
   }

   public Iterator iterator() {
      return sharedFiles.iterator();
   }

   public String hitsToXML(String regex) {
      Document     doc;
      Element      rootElement;
      Element      fileElement;
      Element      nameElement;
      Element      sizeElement;
      Element      digestElement;
      SharedFile   shared;
      RE           re;
      int          nFiles;
      int          filesMax = 5;

      rootElement = new Element("sharing");
      doc         = new Document(rootElement);

      try {
         re     = new RE(regex, RE.MATCH_CASEINDEPENDENT);
         nFiles = 0;
         for( Iterator i = sharedFiles.iterator(); i.hasNext(); ) {

            // a UDP packet can't encapsulate more than 4 filespecs
               shared = (SharedFile )i.next();
               if( re.match(shared.fileName) ) {
                  if( ++nFiles < filesMax ) {
                     nameElement   = new Element("name");
                     nameElement.addContent(shared.fileName);
      
                     sizeElement   = new Element("size");
                     sizeElement.addContent(Integer.toString(shared.size));
      
                     digestElement = new Element("digest");
                     digestElement.addContent(shared.digest);
      
                     fileElement   = new Element("file");
                     fileElement.addContent(nameElement);
                     fileElement.addContent(sizeElement);
                     fileElement.addContent(digestElement);
      
                     rootElement.addContent(fileElement);
                  }
                  else break;
               }
         }
      }
      catch (RESyntaxException e) {
         logger.error("RE syntax exception:  " + regex);   
      }
      return new XMLOutputter(Format.getPrettyFormat()).outputString(doc);
   }

   public boolean isShared(String digest) {
      boolean shared = false;
      
      for(Iterator i = sharedFiles.iterator(); i.hasNext(); ) {
         SharedFile f;
         f = (SharedFile )i.next();
         if( f.digest.equals(digest)) {
            shared = true;
            break;
         }
      }
      return shared;
   }

   public String getFileNameForDigest(String digest) {
      SharedFile f;
      String     fileName = null;
      
      for(Iterator i = sharedFiles.iterator(); i.hasNext(); ) {         
         f = (SharedFile )i.next();
         if( f.digest.equals(digest)) {
            fileName = f.getName();
            break;
         }
      }
      return fileName;
   }

   public void add(SharedFile sf) {
      sharedFiles.add(sf);
   }

   public void remove(String fileName) {
      SharedFile n;
      SharedFile target;

      n = null;
      for( Iterator i = sharedFiles.iterator(); i.hasNext(); ) {
         target = (SharedFile )i.next();
         if( target.getName().equals(fileName) ) {
            n = target;
            break;
         }
      }
      if( n != null ) {
         try {

            // when called as a result of fspClient.download() throwing 
            // an exception, this delete() may be the only significant
            // loc in this method that gets called.
            (new File(sharedDirectory + "/" + fileName)).delete();
         }
         catch (java.lang.SecurityException e) {
            e.printStackTrace();
         }
         sharedFiles.remove(n);
      }
   }
   
   // Test harness
   public static void main(String[] argv) {
      System.out.println(new ShareList("share/flux").hitsToXML(argv[0]));
   }
}
